#include "SynergiesModel.h"
#include "TrajOpt.h"
#include <opencv2/opencv.hpp>

SynergiesModel::SynergiesModel(double noise, int nbr_pcs, int maxIter, int lambda, int STEPS, std::vector<double> initial_coords, std::vector<double> end_eff_target, std::string model_file, std::string matrix_file, int DIM_REDUCTION)
{	
	TrajOpt trajopt(model_file);

	int nlines = count_number_of_lines(matrix_file);	
	int ncolumns = trajopt.get_numControls();
	int nrows = nlines/ncolumns;
	
	cv::Mat M(nrows, ncolumns, CV_64F);
	
	if(std::ifstream(matrix_file.c_str())){
		std::ifstream ifile(matrix_file.c_str());
		for(int i = 0; i < nrows; i++){
			for(int j = 0; j < ncolumns; j++){
				ifile >> M.at<double>(i,j);
			}
		}
		ifile.close();		
	}
	
	cv::Mat H;
	cv::PCA pca;
	cv::Mat mean;
	cv::Mat eigenvalues;
	cv::Mat ev;

	if(DIM_REDUCTION == 0){
		std::string nmf_file = "../data/H";	
	
		if(std::ifstream(nmf_file.c_str())){
			std::ifstream nmf_ifile(nmf_file.c_str());
			int nmf_rows, nmf_cols;   
			nmf_ifile >> nmf_rows;  
			nmf_ifile >> nmf_cols;  
			H = cv::Mat(nmf_rows, nmf_cols, CV_64F);	
			for(int i = 0; i < nmf_rows; i++){
				for(int j = 0; j < nmf_cols; j++){
					nmf_ifile >> H.at<double>(i,j);
				}
			}
			nmf_ifile.close();	
		}
	}else{		
		pca = cv::PCA(M, cv::Mat(), cv::PCA::DATA_AS_ROW, nbr_pcs);
		mean = pca.mean.clone();
    		eigenvalues = pca.eigenvalues.clone();
    		ev = pca.eigenvectors.clone();
	}
		
	TrajOpt topt_stroke(model_file);
	
	if(DIM_REDUCTION == 0){
		nbr_pcs = H.rows;
		topt_stroke.set_low_dim_params(nbr_pcs, H, H);
		topt_stroke.set_lower_upper_bound(-0.3, 1.0);
		topt_stroke.set_init_stepsize(0.1);
	}else{
		topt_stroke.set_low_dim_params(nbr_pcs, ev, mean);
		topt_stroke.set_lower_upper_bound(-1.0, 1.0);
		topt_stroke.set_init_stepsize(0.3);
	}
	topt_stroke.set_app_filename("ue_low_stroke");
	topt_stroke.set_maxIter(maxIter);
	topt_stroke.set_lambda(lambda);
	topt_stroke.set_NOISE(noise);
	topt_stroke.set_STEP(STEPS);

	if(end_eff_target[0] != SimTK::Infinity){
		topt_stroke.set_random_initial_state_end_eff(initial_coords, end_eff_target);
	}else{
		topt_stroke.set_initial_state(initial_coords);
	}
	topt_stroke.set_dim_reduction(DIM_REDUCTION);
	topt_stroke.traj_opt_setup();

	_numControls = nbr_pcs; 
	_control_solutions = topt_stroke.get_controls();	
	_control_list = topt_stroke.get_control_list();
	_state_list = topt_stroke.get_state_list(); 
	_bestSoFar = topt_stroke.get_bestSoFar();	
}

int SynergiesModel::count_number_of_lines(std::string filename)
{
	std::ifstream f(filename.c_str());
	std::string line;
	int nrows;
	
	for(nrows = 0; std::getline(f,line); nrows++)
		;		
	f.close();
	return nrows;	
}


