#include "WorkspaceOptimization.h" 
#include "TrajOpt.h"
#include <opencv2/opencv.hpp>
#include <mpi.h>

void WorkspaceOptimization::optimize(int NBR_TOPT, std::string model_file, std::string matrix_file)
{
	int nbr_rows = 0;	
	cv::Mat M;			

	std::ofstream ofile;
	bool APPEND = false;
	
	int numtasks = 0;
	int myrank = 0;
	MPI_Comm_size(MPI_COMM_WORLD,&numtasks);
	MPI_Comm_rank(MPI_COMM_WORLD,&myrank);
	
	for (int k = 0; k < NBR_TOPT; k++){	
	
		TrajOpt trajopt(model_file);
		int numParams = trajopt.get_numControls();
		trajopt.set_maxIter(_maxIter);
		trajopt.set_lambda(_lambda);	
		if(!_randomInitial){
			trajopt.set_initial_state(_initial_coords);
		}
		trajopt.set_lower_upper_bound(-10.0,10.0);		
		trajopt.traj_opt_setup();
			
		SimTK::Vector control_solutions = trajopt.get_controls();
		
		int start_row = nbr_rows;	
		nbr_rows = nbr_rows + control_solutions.size()/numParams;	
		
		if (k == 0){
			M = cv::Mat(nbr_rows, numParams, cv::DataType<double>::type);	
		}else{
			M.resize(nbr_rows);
		}
	
		int row = 0, column = 0;
		for(int i = 0; i < control_solutions.size(); i++){
			row = (int)i/numParams;
			column = i%numParams;
			M.at<double>(start_row + row,column) = control_solutions.get(i);
		}		
	
		if(myrank == 0){
			if(!_randomInitial){
				std::string initial_state_file = matrix_file + "_initial_state";
				ofile.open(initial_state_file.c_str(), std::fstream::out);
				for(int i = 0; i < _initial_coords.size(); i++){
					ofile << _initial_coords.at(i) << std::endl;
				}	
				ofile.close();
			}

			ofile.open(matrix_file.c_str(), std::fstream::in | std::fstream::out | std::fstream::app);
			for(int i = start_row; i < M.rows; i++){
				for(int j = 0; j < M.cols; j++){
					ofile << std::fixed << std::setprecision(std::numeric_limits<double>::digits10+2) << M.at<double>(i,j) << std::endl;			
				}
			}
			ofile.close();
		}	
	}
}

WorkspaceOptimization::WorkspaceOptimization(int maxIter, int lambda, int NBR_TOPT, std::string model_file, std::string matrix_file)
{
	_randomInitial = true;
	_maxIter = maxIter;
	_lambda = lambda;
	optimize(NBR_TOPT, model_file, matrix_file);	
}

WorkspaceOptimization::WorkspaceOptimization(int maxIter, int lambda, int NBR_TOPT, std::string model_file, std::string matrix_file, std::vector<double> initial_coords)
{
	_randomInitial = false;
	_maxIter = maxIter;
	_lambda = lambda;
	_initial_coords = initial_coords;
	optimize(NBR_TOPT, model_file, matrix_file);	
}
